# commis.program
# Implements a complete console program.
#
# Author:   Benjamin Bengfort <benjamin@bengfort.com>
# Created:  Wed Jan 20 14:02:11 2016 -0500
#
# Copyright (C) 2016 Bengfort.com
# For license information, see LICENSE.txt
#
# ID: program.py [] benjamin@bengfort.com $

"""
Implements a complete console program.
"""

##########################################################################
## Imports
##########################################################################

import os
import sys

import argparse
import traceback


##########################################################################
## Helper functions
##########################################################################

def handle_default_args(args):
    """
    Include handling of any default arguments that all commands should
    implement here (for example, specifying the pythonpath).
    """

    if hasattr(args, 'pythonpath'):
        if args.pythonpath:
            sys.path.insert(0, args.pythonpath)

##########################################################################
## Console Program
##########################################################################

class ConsoleProgram(object):
    """
    The base program from which all console commands are derived.
    """

    description = None
    epilog      = None
    version     = None

    def __init__(self, **kwargs):
        self.version     = kwargs.get('version', self.__class__.version)
        self.description = kwargs.get('description', self.__class__.description)
        self.epilog      = kwargs.get('epilog', self.__class__.epilog)
        self.commands    = {} # Stores the subcommands of the program

        self._parser     = None
        self._subparsers = None

    @property
    def parser(self):
        """
        Instantiates the argparse parser
        """
        if self._parser is None:
            apkw = {
                'description': self.description,
                'epilog':      self.epilog,
                'version':     self.version,
            }
            self._parser = argparse.ArgumentParser(**apkw)
        return self._parser

    @property
    def subparsers(self):
        """
        Insantiates the subparsers for all commands
        """
        if self._subparsers is None:
            apkw = {
                'title': 'commands',
                'description': 'Commands for the %s program' % self.parser.prog,
            }
            self._subparsers = self.parser.add_subparsers(**apkw)
        return self._subparsers

    def register(self, command):
        """
        Registers a command with the program
        """
        command = command()
        if command.name in self.commands:
            raise ValueError("Command %s already registered!" % command.name)

        command.create_parser(self.subparsers)
        self.commands[command.name] = command

    def exit(self, code=0, message=None):
        """
        Exit the console program sanely.
        """

        ## If we have a parser, use it to exit
        if self._parser:
            if code > 0:
                self.parser.error(message)
            else:
                self.parser.exit(code, message)

        ## Else we are exiting before parser creation
        else:
            if message is not None:
                if code > 0:
                    sys.stderr.write(message)
                else:
                    sys.stdout.write(message)
            sys.exit(code)

        ## If we're here we didn't exit for some reason?
        raise Exception("Unable to exit the %s" % self.__class__.__name__)

    def execute(self):
        """
        Entry point to the execution of the program.
        """

        # Handle input from the command line
        args = self.parser.parse_args()                # Parse the arguments

        try:
            if not hasattr(args, 'func'):
                raise NotImplementedError("No commands registered with this program!")

            handle_default_args(args)                  # Handle the default args
            msg = "{}\n".format(args.func(args))       # Call the default function
            self.parser.exit(0, msg)                   # Exit cleanly with message

        except Exception as e:
            if hasattr(args, 'traceback') and args.traceback:
                traceback.print_exc()
            self.parser.error(str(e))                  # Exit with error
